import 'package:test/test.dart';
import 'package:command/command.dart';

main() {
  test("Undo and redo work correctly", () {
    var command = new Command(new CommandActionTester());
    expect(command.canUndo, equals(false));
    expect(command.canExecute, equals(true));

    command.execute();
    expect(command.canUndo, equals(true));
    expect(command.canExecute, equals(false));

    command.undo();
    expect(command.canUndo, false);
    expect(command.canExecute, true);

    command.execute();
    expect(command.canUndo, true);
    expect(command.canExecute, false);
  });

  test("Performing redo when unable to redo throws an error", () {
    var command = new Command(new CommandActionTester())..execute();
    expect(
      () => command.execute(),
      throwsA(new isInstanceOf<StateError>()),
    );
  });

  test("Performing undo when unable to undo throws an error", () {
    var command = new Command(new CommandActionTester());
    expect(
      () => command.undo(),
      throwsA(new isInstanceOf<StateError>()),
    );
  });

  test("Execute and undo chained commands", () {
    var childCommand = new Command(new CommandActionTester());
    var parentCommand = new Command(new CommandActionTester(),
        child: childCommand,
    );

    parentCommand.execute();

    expect(parentCommand.canUndo, isTrue);
    expect(childCommand.canUndo, isTrue);
    expect(parentCommand.canExecute, isFalse);
    expect(childCommand.canExecute, isFalse);

    parentCommand.undo();

    expect(parentCommand.canUndo, isFalse);
    expect(childCommand.canUndo, isFalse);
    expect(parentCommand.canExecute, isTrue);
    expect(childCommand.canExecute, isTrue);
  });

  test("Chained commands execute after and undo before the parent",(){
    var log = new List<String>();
    var parent = new CommandActionOrderTester(log, "Parent");
    var child = new CommandActionOrderTester(log, "Child");

    var command = new Command(parent,child: new Command(child));
    command.execute();
    command.undo();

    expect(log.length,equals(4));
    expect(log[0], equals("Parent execute"));
    expect(log[1], equals("Child execute"));
    expect(log[2], equals("Child undo"));
    expect(log[3], equals("Parent undo"));
  });
}

class CommandActionTester extends CommandAction {
  void action() {}
  void undoAction() {}
  String get key => "CommandActionTester";
}

class CommandActionOrderTester extends CommandAction {
  CommandActionOrderTester(this.log, this.key);

  final List<String> log;
  final String key;

  void action(){
    log.add("$key execute");
  }

  void undoAction(){
    log.add("$key undo");
  }
}
